@@ -33,7 +33,6 @@
 #include <signal.h>
 
 #include "keyserver_x11.h"
-#include "fcitx-compose-data.h"
 
 #include "qfcitxplatforminputcontext.h"
 #include "fcitxqtinputcontextproxy.h"
@@ -42,41 +41,6 @@
 #include "keyuni.h"
 #include "utils.h"
 
-typedef struct _FcitxComposeTableCompact FcitxComposeTableCompact;
-struct _FcitxComposeTableCompact {
-    const quint32 *data;
-    int max_seq_len;
-    int n_index_size;
-    int n_index_stride;
-};
-
-static const FcitxComposeTableCompact fcitx_compose_table_compact = {
-    fcitx_compose_seqs_compact,
-    5,
-    23,
-    6
-};
-
-static const uint fcitx_compose_ignore[] = {
-    XK_Shift_L,
-    XK_Shift_R,
-    XK_Control_L,
-    XK_Control_R,
-    XK_Caps_Lock,
-    XK_Shift_Lock,
-    XK_Meta_L,
-    XK_Meta_R,
-    XK_Alt_L,
-    XK_Alt_R,
-    XK_Super_L,
-    XK_Super_R,
-    XK_Hyper_L,
-    XK_Hyper_R,
-    XK_Mode_switch,
-    XK_ISO_Level3_Shift,
-    XK_VoidSymbol
-};
-
 static bool key_filtered = false;
 
 static bool
@@ -85,7 +49,7 @@ get_boolean_env(const char *name,
 {
     const char *value = getenv(name);
 
-    if (value == NULL)
+    if (value == nullptr)
         return defval;
 
     if (strcmp(value, "") == 0 ||
@@ -98,51 +62,44 @@ get_boolean_env(const char *name,
     return true;
 }
 
-static int
-compare_seq_index(const void *key, const void *value)
+static inline const char*
+get_locale()
 {
-    const uint *keysyms = (const uint *)key;
-    const quint32 *seq = (const quint32 *)value;
+    const char* locale = getenv("LC_ALL");
+    if (!locale)
+        locale = getenv("LC_CTYPE");
+    if (!locale)
+        locale = getenv("LANG");
+    if (!locale)
+        locale = "C";
 
-    if (keysyms[0] < seq[0])
-        return -1;
-    else if (keysyms[0] > seq[0])
-        return 1;
-    return 0;
+    return locale;
 }
 
-static int
-compare_seq(const void *key, const void *value)
+struct xkb_context* _xkb_context_new_helper()
 {
-    int i = 0;
-    const uint *keysyms = (const uint *)key;
-    const quint32 *seq = (const quint32 *)value;
-
-    while (keysyms[i]) {
-        if (keysyms[i] < seq[i])
-            return -1;
-        else if (keysyms[i] > seq[i])
-            return 1;
-        i++;
+    struct xkb_context* context = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
+    if (context) {
+        xkb_context_set_log_level(context, XKB_LOG_LEVEL_CRITICAL);
     }
 
-    return 0;
+    return context;
 }
 
-
 QFcitxPlatformInputContext::QFcitxPlatformInputContext() :
     m_connection(new FcitxQtConnection(this)),
-    m_improxy(0),
-    m_n_compose(0),
+    m_improxy(nullptr),
     m_cursorPos(0),
     m_useSurroundingText(false),
-    m_syncMode(true),
+    m_syncMode(get_boolean_env("FCITX_QT_USE_SYNC", false)),
     m_lastWId(0),
-    m_destroy(false)
+    m_destroy(false),
+    m_xkbContext(_xkb_context_new_helper()),
+    m_xkbComposeTable(m_xkbContext ? xkb_compose_table_new_from_locale(m_xkbContext.data(), get_locale(), XKB_COMPOSE_COMPILE_NO_FLAGS) : 0),
+    m_xkbComposeState(m_xkbComposeTable ? xkb_compose_state_new(m_xkbComposeTable.data(), XKB_COMPOSE_STATE_NO_FLAGS) : 0)
 {
     FcitxQtFormattedPreedit::registerMetaType();
 
-    memset(m_compose_buffer, 0, sizeof(uint) * (MAX_COMPOSE_LEN + 1));
     connect(m_connection, SIGNAL(connected()), this, SLOT(connected()));
     connect(m_connection, SIGNAL(disconnected()), this, SLOT(cleanUp()));
 
@@ -163,7 +120,7 @@ void QFcitxPlatformInputContext::connected()
     // qDebug() << "create Input Context" << m_connection->name();
     if (m_improxy) {
         delete m_improxy;
-        m_improxy = 0;
+        m_improxy = nullptr;
     }
     m_improxy = new FcitxQtInputMethodProxy(m_connection->serviceName(), QLatin1String("/inputmethod"), *m_connection->connection(), this);
 
@@ -174,19 +131,11 @@ void QFcitxPlatformInputContext::connected()
 
 void QFcitxPlatformInputContext::cleanUp()
 {
-    for(QHash<WId, FcitxQtICData *>::const_iterator i = m_icMap.constBegin(),
-                                             e = m_icMap.constEnd(); i != e; ++i) {
-        FcitxQtICData* data = i.value();
-
-        if (data->proxy)
-            delete data->proxy;
-    }
-
     m_icMap.clear();
 
     if (m_improxy) {
         delete m_improxy;
-        m_improxy = 0;
+        m_improxy = nullptr;
     }
 
     if (!m_destroy) {
@@ -230,6 +179,9 @@ void QFcitxPlatformInputContext::reset()
     FcitxQtInputContextProxy* proxy = validIC();
     if (proxy)
         proxy->Reset();
+    if (m_xkbComposeState) {
+        xkb_compose_state_reset(m_xkbComposeState.data());
+    }
     QPlatformInputContext::reset();
 }
 
@@ -240,9 +192,8 @@ void QFcitxPlatformInputContext::update(Qt::InputMethodQueries queries )
     if (!proxy)
         return;
 
-    FcitxQtICData* data = m_icMap.value(window->winId());
+    auto &data = m_icMap[window->winId()];
 
-    QInputMethod *method = qApp->inputMethod();
     QObject *input = qApp->focusObject();
     if (!input)
         return;
@@ -283,7 +234,7 @@ void QFcitxPlatformInputContext::update(Qt::InputMethodQueries queries )
             break;
         if (!((queries & Qt::ImSurroundingText) && (queries & Qt::ImCursorPosition)))
             break;
-        if (data->capacity.testFlag(CAPACITY_PASSWORD))
+        if (data.capacity.testFlag(CAPACITY_PASSWORD))
             break;
         QVariant var = query.value(Qt::ImSurroundingText);
         QVariant var1 = query.value(Qt::ImCursorPosition);
@@ -303,24 +254,24 @@ void QFcitxPlatformInputContext::update(Qt::InputMethodQueries queries )
                     anchor = var2.toInt();
                 else
                     anchor = cursor;
-                if (data->surroundingText != text) {
-                    data->surroundingText = text;
+                if (data.surroundingText != text) {
+                    data.surroundingText = text;
                     proxy->SetSurroundingText(text, cursor, anchor);
                 }
                 else {
-                    if (data->surroundingAnchor != anchor ||
-                        data->surroundingCursor != cursor)
+                    if (data.surroundingAnchor != anchor ||
+                        data.surroundingCursor != cursor)
                         proxy->SetSurroundingTextPosition(cursor, anchor);
                 }
-                data->surroundingCursor = cursor;
-                data->surroundingAnchor = anchor;
+                data.surroundingCursor = cursor;
+                data.surroundingAnchor = anchor;
                 setSurrounding = true;
             }
         }
         if (!setSurrounding) {
-            data->surroundingAnchor = -1;
-            data->surroundingCursor = -1;
-            data->surroundingText = QString::null;
+            data.surroundingAnchor = -1;
+            data.surroundingCursor = -1;
+            data.surroundingText = QString::null;
             removeCapacity(data, CAPACITY_SURROUNDING_TEXT);
         }
     } while(0);
@@ -349,25 +300,19 @@ void QFcitxPlatformInputContext::setFocusObject(QObject* object)
     if (proxy)
         proxy->FocusIn();
     else {
-        FcitxQtICData* data = m_icMap.value(window->winId());
-        if (!data) {
-            createICData(window);
-            return;
-        }
+        createICData(window);
     }
 }
 
 void QFcitxPlatformInputContext::windowDestroyed(QObject* object)
 {
     /* access QWindow is not possible here, so we use our own map to do so */
-    WId wid = m_windowToWidMap.take(object);
-    if (!wid)
+    auto iter = m_windowToWidMap.find(object);
+    if (iter == m_windowToWidMap.end())
         return;
-    FcitxQtICData* data = m_icMap.take(wid);
-    if (!data)
-        return;
-
-    delete data;
+    auto wid = iter->second;
+    m_windowToWidMap.erase(iter);
+    m_icMap.erase(wid);
     // qDebug() << "Window Destroyed and we destroy IC correctly, horray!";
 }
 
@@ -380,7 +325,7 @@ void QFcitxPlatformInputContext::cursorRectChanged()
     if (!proxy)
         return;
 
-    FcitxQtICData* data = m_icMap.value(inputWindow->winId());
+    auto& data = m_icMap[inputWindow->winId()];
 
     QRect r = qApp->inputMethod()->cursorRectangle().toRect();
     if(!r.isValid())
@@ -388,9 +333,11 @@ void QFcitxPlatformInputContext::cursorRectChanged()
 
     r.moveTopLeft(inputWindow->mapToGlobal(r.topLeft()));
 
-    if (data->rect != r) {
-        data->rect = r;
-        proxy->SetCursorRect(r.x(), r.y(), r.width(), r.height());
+    qreal scale = inputWindow->devicePixelRatio();
+    if (data.rect != r) {
+        data.rect = r;
+        proxy->SetCursorRect(r.x() * scale, r.y() * scale,
+                             r.width() * scale, r.height() * scale);
     }
 }
 
@@ -401,11 +348,9 @@ void QFcitxPlatformInputContext::createInputContext(WId w)
 
     // qDebug() << "create Input Context" << m_connection->connection()->name();
 
-    if (m_improxy) {
-        delete m_improxy;
-        m_improxy = NULL;
+    if (!m_improxy) {
+        m_improxy = new FcitxQtInputMethodProxy(m_connection->serviceName(), QLatin1String("/inputmethod"), *m_connection->connection(), this);
     }
-    m_improxy = new FcitxQtInputMethodProxy(m_connection->serviceName(), QLatin1String("/inputmethod"), *m_connection->connection(), this);
 
     if (!m_improxy->isValid())
         return;
@@ -420,9 +365,12 @@ void QFcitxPlatformInputContext::createInputContext(WId w)
 void QFcitxPlatformInputContext::createInputContextFinished(QDBusPendingCallWatcher* watcher)
 {
     WId w = watcher->property("wid").toULongLong();
-    FcitxQtICData* data = m_icMap.value(w);
-    if (!data)
+    auto iter = m_icMap.find(w);
+    if (iter == m_icMap.end()) {
         return;
+    }
+
+    auto &data = iter->second;
 
     QDBusPendingReply< int, bool, uint, uint, uint, uint > result = *watcher;
 
@@ -436,45 +384,42 @@ void QFcitxPlatformInputContext::createInputContextFinished(QDBusPendingCallWatc
 
         int id = qdbus_cast<int>(result.argumentAt(0));
         QString path = QString("/inputcontext_%1").arg(id);
-        if (data->proxy) {
-            delete data->proxy;
+        if (data.proxy) {
+            delete data.proxy;
         }
-        data->proxy = new FcitxQtInputContextProxy(m_connection->serviceName(), path, *m_connection->connection(), this);
-        connect(data->proxy, SIGNAL(CommitString(QString)), this, SLOT(commitString(QString)));
-        connect(data->proxy, SIGNAL(ForwardKey(uint, uint, int)), this, SLOT(forwardKey(uint, uint, int)));
-        connect(data->proxy, SIGNAL(UpdateFormattedPreedit(FcitxQtFormattedPreeditList,int)), this, SLOT(updateFormattedPreedit(FcitxQtFormattedPreeditList,int)));
-        connect(data->proxy, SIGNAL(DeleteSurroundingText(int,uint)), this, SLOT(deleteSurroundingText(int,uint)));
+        data.proxy = new FcitxQtInputContextProxy(m_connection->serviceName(), path, *m_connection->connection(), this);
+        connect(data.proxy, SIGNAL(CommitString(QString)), this, SLOT(commitString(QString)));
+        connect(data.proxy, SIGNAL(ForwardKey(uint, uint, int)), this, SLOT(forwardKey(uint, uint, int)));
+        connect(data.proxy, SIGNAL(UpdateFormattedPreedit(FcitxQtFormattedPreeditList,int)), this, SLOT(updateFormattedPreedit(FcitxQtFormattedPreeditList,int)));
+        connect(data.proxy, SIGNAL(DeleteSurroundingText(int,uint)), this, SLOT(deleteSurroundingText(int,uint)));
+        connect(data.proxy, SIGNAL(CurrentIM(QString,QString,QString)), this, SLOT(updateCurrentIM(QString,QString,QString)));
 
-        if (data->proxy->isValid()) {
+        if (data.proxy->isValid()) {
             QWindow* window = qApp->focusWindow();
             if (window && window->winId() == w)
-                data->proxy->FocusIn();
+                data.proxy->FocusIn();
         }
 
         QFlags<FcitxCapacityFlags> flag;
         flag |= CAPACITY_PREEDIT;
         flag |= CAPACITY_FORMATTED_PREEDIT;
         flag |= CAPACITY_CLIENT_UNFOCUS_COMMIT;
+        flag |= CAPACITY_GET_IM_INFO_ON_FOCUS;
         m_useSurroundingText = get_boolean_env("FCITX_QT_ENABLE_SURROUNDING_TEXT", true);
         if (m_useSurroundingText)
             flag |= CAPACITY_SURROUNDING_TEXT;
 
-        /*
-         * event loop will cause some problem, so we tries to use async way.
-         */
-        m_syncMode = get_boolean_env("FCITX_QT_USE_SYNC", false);
-
         addCapacity(data, flag, true);
     } while(0);
     delete watcher;
 }
 
-void QFcitxPlatformInputContext::updateCapacity(FcitxQtICData* data)
+void QFcitxPlatformInputContext::updateCapacity(const FcitxQtICData &data)
 {
-    if (!data->proxy || !data->proxy->isValid())
+    if (!data.proxy || !data.proxy->isValid())
         return;
 
-    QDBusPendingReply< void > result = data->proxy->SetCapacity((uint) data->capacity);
+    QDBusPendingReply< void > result = data.proxy->SetCapacity((uint) data.capacity);
 }
 
 void QFcitxPlatformInputContext::commitString(const QString& str)
@@ -549,7 +494,7 @@ void QFcitxPlatformInputContext::deleteSurroundingText(int offset, uint nchar)
 void QFcitxPlatformInputContext::forwardKey(uint keyval, uint state, int type)
 {
     QObject *input = qApp->focusObject();
-    if (input != NULL) {
+    if (input != nullptr) {
         key_filtered = true;
         QKeyEvent *keyevent = createKeyEvent(keyval, state, type);
         QCoreApplication::sendEvent(input, keyevent);
@@ -558,12 +503,27 @@ void QFcitxPlatformInputContext::forwardKey(uint keyval, uint state, int type)
     }
 }
 
+void QFcitxPlatformInputContext::updateCurrentIM(const QString& name, const QString& uniqueName, const QString& langCode)
+{
+    Q_UNUSED(name);
+    Q_UNUSED(uniqueName);
+    QLocale newLocale(langCode);
+    if (m_locale != newLocale) {
+        m_locale = newLocale;
+        emitLocaleChanged();
+    }
+}
+
+QLocale QFcitxPlatformInputContext::locale() const
+{
+    return m_locale;
+}
+
 void QFcitxPlatformInputContext::createICData(QWindow* w)
 {
-    FcitxQtICData* data = m_icMap.value(w->winId());
-    if (!data) {
-        data = new FcitxQtICData;
-        m_icMap[w->winId()] = data;
+    auto iter = m_icMap.find(w->winId());
+    if (iter == m_icMap.end()) {
+        m_icMap.emplace(std::piecewise_construct, std::forward_as_tuple(w->winId()), std::forward_as_tuple());
         m_windowToWidMap[w] = w->winId();
         connect(w, SIGNAL(destroyed(QObject*)), this, SLOT(windowDestroyed(QObject*)));
     }
@@ -717,12 +677,9 @@ void QFcitxPlatformInputContext::processKeyEventFinished(QDBusPendingCallWatcher
         }
         QWindowSystemInterface::handleExtendedKeyEvent(window, time, type, qtcode, modifiers,
                                                        code, sym, state, string, isAutoRepeat);
-
-        if (isAutoRepeat && type == QEvent::KeyRelease) {
-            QWindowSystemInterface::handleExtendedKeyEvent(window, time, QEvent::KeyPress, qtcode, modifiers,
-                                                           code, sym, state, string, isAutoRepeat);
-        }
     }
+
+    delete watcher;
 }
 
 
@@ -737,7 +694,7 @@ bool QFcitxPlatformInputContext::filterEventFallback(uint keyval, uint keycode,
 
 FcitxQtInputContextProxy* QFcitxPlatformInputContext::validIC()
 {
-    if (m_icMap.isEmpty()) {
+    if (m_icMap.empty()) {
         return 0;
     }
     QWindow* window = qApp->focusWindow();
@@ -746,18 +703,19 @@ FcitxQtInputContextProxy* QFcitxPlatformInputContext::validIC()
 
 FcitxQtInputContextProxy* QFcitxPlatformInputContext::validICByWId(WId wid)
 {
-    if (m_icMap.isEmpty()) {
-        return 0;
+    if (m_icMap.empty()) {
+        return nullptr;
     }
-    FcitxQtICData* icData = m_icMap.value(wid);
-    if (!icData)
-        return 0;
-    if (icData->proxy.isNull()) {
-        return 0;
-    } else if (icData->proxy->isValid()) {
-        return icData->proxy.data();
+    auto iter = m_icMap.find(wid);
+    if (iter == m_icMap.end())
+        return nullptr;
+    auto &data = iter->second;
+    if (data.proxy.isNull()) {
+        return nullptr;
+    } else if (data.proxy->isValid()) {
+        return data.proxy.data();
     }
-    return 0;
+    return nullptr;
 }
 
 FcitxQtInputContextProxy* QFcitxPlatformInputContext::validICByWindow(QWindow* w)
@@ -766,7 +724,7 @@ FcitxQtInputContextProxy* QFcitxPlatformInputContext::validICByWindow(QWindow* w
         return 0;
     }
 
-    if (m_icMap.isEmpty()) {
+    if (m_icMap.empty()) {
         return 0;
     }
     return validICByWId(w->winId());
@@ -776,195 +734,34 @@ FcitxQtInputContextProxy* QFcitxPlatformInputContext::validICByWindow(QWindow* w
 bool QFcitxPlatformInputContext::processCompose(uint keyval, uint state, FcitxKeyEventType event)
 {
     Q_UNUSED(state);
-    int i;
 
-    if (event == FCITX_RELEASE_KEY)
+    if (!m_xkbComposeTable || event == FCITX_RELEASE_KEY)
         return false;
 
-    for (i = 0; fcitx_compose_ignore[i] != XK_VoidSymbol; i++) {
-        if (keyval == fcitx_compose_ignore[i])
-            return false;
-    }
+    struct xkb_compose_state* xkbComposeState = m_xkbComposeState.data();
 
-    m_compose_buffer[m_n_compose ++] = keyval;
-    m_compose_buffer[m_n_compose] = 0;
-
-    if (checkCompactTable(&fcitx_compose_table_compact)) {
-        // qDebug () << "checkCompactTable ->true";
-        return true;
-    }
-
-    if (checkAlgorithmically()) {
-        // qDebug () << "checkAlgorithmically ->true";
-        return true;
-    }
-
-    if (m_n_compose > 1) {
-        m_compose_buffer[0] = 0;
-        m_n_compose = 0;
-        return true;
-    } else {
-        m_compose_buffer[0] = 0;
-        m_n_compose = 0;
+    enum xkb_compose_feed_result result = xkb_compose_state_feed(xkbComposeState, keyval);
+    if (result == XKB_COMPOSE_FEED_IGNORED) {
         return false;
     }
+
+    enum xkb_compose_status status = xkb_compose_state_get_status(xkbComposeState);
+    if (status == XKB_COMPOSE_NOTHING) {
+        return 0;
+    } else if (status == XKB_COMPOSE_COMPOSED) {
+        char buffer[] = {'\0', '\0', '\0', '\0', '\0', '\0', '\0'};
+        int length = xkb_compose_state_get_utf8(xkbComposeState, buffer, sizeof(buffer));
+        xkb_compose_state_reset(xkbComposeState);
+        if (length != 0) {
+            commitString(QString::fromUtf8(buffer));
+        }
+
+    } else if (status == XKB_COMPOSE_CANCELLED) {
+        xkb_compose_state_reset(xkbComposeState);
+    }
+
+    return true;
 }
 
-#define IS_DEAD_KEY(k) \
-    ((k) >= XK_dead_grave && (k) <= (XK_dead_dasia+1))
-
-bool QFcitxPlatformInputContext::checkAlgorithmically()
-{
-    int i;
-    quint32 combination_buffer[MAX_COMPOSE_LEN];
-
-    if (m_n_compose >= MAX_COMPOSE_LEN)
-        return false;
-
-    for (i = 0; i < m_n_compose && IS_DEAD_KEY(m_compose_buffer[i]); i++);
-    if (i == m_n_compose)
-        return true;
-
-    if (i > 0 && i == m_n_compose - 1) {
-        combination_buffer[0] = FcitxKeySymToUnicode(m_compose_buffer[i]);
-        combination_buffer[m_n_compose] = 0;
-        i--;
-        while (i >= 0) {
-            switch (m_compose_buffer[i]) {
-#define CASE(keysym, unicode) \
-case XK_dead_##keysym: combination_buffer[i + 1] = unicode; break
-                CASE(grave, 0x0300);
-                CASE(acute, 0x0301);
-                CASE(circumflex, 0x0302);
-                CASE(tilde, 0x0303);  /* Also used with perispomeni, 0x342. */
-                CASE(macron, 0x0304);
-                CASE(breve, 0x0306);
-                CASE(abovedot, 0x0307);
-                CASE(diaeresis, 0x0308);
-                CASE(hook, 0x0309);
-                CASE(abovering, 0x030A);
-                CASE(doubleacute, 0x030B);
-                CASE(caron, 0x030C);
-                CASE(abovecomma, 0x0313);  /* Equivalent to psili */
-                CASE(abovereversedcomma, 0x0314);  /* Equivalent to dasia */
-                CASE(horn, 0x031B);  /* Legacy use for psili, 0x313 (or 0x343). */
-                CASE(belowdot, 0x0323);
-                CASE(cedilla, 0x0327);
-                CASE(ogonek, 0x0328);  /* Legacy use for dasia, 0x314.*/
-                CASE(iota, 0x0345);
-                CASE(voiced_sound, 0x3099);  /* Per Markus Kuhn keysyms.txt file. */
-                CASE(semivoiced_sound, 0x309A);  /* Per Markus Kuhn keysyms.txt file. */
-                /* The following cases are to be removed once xkeyboard-config,
-                * xorg are fully updated.
-                **/
-                /* Workaround for typo in 1.4.x xserver-xorg */
-            case 0xfe66:
-                combination_buffer[i + 1] = 0x314;
-                break;
-                /* CASE (dasia, 0x314); */
-                /* CASE (perispomeni, 0x342); */
-                /* CASE (psili, 0x343); */
-#undef CASE
-            default:
-                combination_buffer[i + 1] = FcitxKeySymToUnicode(m_compose_buffer[i]);
-            }
-            i--;
-        }
-
-        /* If the buffer normalizes to a single character,
-        * then modify the order of combination_buffer accordingly, if necessary,
-        * and return TRUE.
-        **/
-#if 0
-        if (check_normalize_nfc(combination_buffer, m_n_compose)) {
-            gunichar value;
-            combination_utf8 = g_ucs4_to_utf8(combination_buffer, -1, NULL, NULL, NULL);
-            nfc = g_utf8_normalize(combination_utf8, -1, G_NORMALIZE_NFC);
-
-            value = g_utf8_get_char(nfc);
-            gtk_im_context_simple_commit_char(GTK_IM_CONTEXT(context_simple), value);
-            context_simple->compose_buffer[0] = 0;
-
-            g_free(combination_utf8);
-            g_free(nfc);
-
-            return TRUE;
-        }
-#endif
-        QString s(QString::fromUcs4(combination_buffer, m_n_compose));
-        s = s.normalized(QString::NormalizationForm_C);
-
-        // qDebug () << "combination_buffer = " << QString::fromUcs4(combination_buffer, m_n_compose) << "m_n_compose" << m_n_compose;
-        // qDebug () << "result = " << s << "i = " << s.length();
-
-        if (s.length() == 1) {
-            commitString(QString(s[0]));
-            m_compose_buffer[0] = 0;
-            m_n_compose = 0;
-            return true;
-        }
-    }
-    return false;
-}
-
-
-bool
-QFcitxPlatformInputContext::checkCompactTable(const FcitxComposeTableCompact *table)
-{
-    int row_stride;
-    const quint32 *seq_index;
-    const quint32 *seq;
-    int i;
-
-    /* Will never match, if the sequence in the compose buffer is longer
-    * than the sequences in the table. Further, compare_seq (key, val)
-    * will overrun val if key is longer than val. */
-    if (m_n_compose > table->max_seq_len)
-        return false;
-
-    seq_index = (const quint32 *)bsearch(m_compose_buffer,
-                                         table->data, table->n_index_size,
-                                         sizeof(quint32) * table->n_index_stride,
-                                         compare_seq_index);
-
-    if (!seq_index) {
-        return false;
-    }
-
-    if (seq_index && m_n_compose == 1) {
-        return true;
-    }
-
-    seq = NULL;
-    for (i = m_n_compose - 1; i < table->max_seq_len; i++) {
-        row_stride = i + 1;
-
-        if (seq_index[i + 1] - seq_index[i] > 0) {
-            seq = (const quint32 *) bsearch(m_compose_buffer + 1,
-                                            table->data + seq_index[i], (seq_index[i + 1] - seq_index[i]) / row_stride,
-                                            sizeof(quint32) * row_stride,
-                                            compare_seq);
-            if (seq) {
-                if (i == m_n_compose - 1)
-                    break;
-                else {
-                    return true;
-                }
-            }
-        }
-    }
-
-    if (!seq) {
-        return false;
-    } else {
-        uint value;
-        value = seq[row_stride - 1];
-        commitString(QString(QChar(value)));
-        m_compose_buffer[0] = 0;
-        m_n_compose = 0;
-        return true;
-    }
-    return false;
-}
 
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
